<%
Function ComputeWeekdayHoursDuration(StartDateTime, EndDateTime)

'  This function expects two parameters as input - a beginning date/time and an ending date/time.
'  On the basis of a table denoting business hours-per-weekday, it computes how many business hours
'  elapsed between the beginning date/time and the ending date/time. It does this with a granularity
'  of quarter hours i.e. the table denoting business hours for each of the 7 days in a week has 96 slots matching
'  the 96 quarter-hours in a 24-hour day. The matching slots in this table are examined for each quarter-hour
'  between beginning date/time and ending date/time to determine whether it should be counted into the final
'  result. "Matching" here means on the basis of the day-of-week (i.e. Mon, Tue, Wed, etc.) and which of the
'  96 quarter-hours in the day
'
'  The logic for this function is divided into 5 main phases:
'
'		1) Initialization, set up the bit vectors and compute totals per day-of-week
'
'		2) Compute how many billable quarter hours are involved on the beginning (partial) day
'
'		3) if begin day <> end day, Compute how many billable quarter hours are involved on the ending (partial) day
'
'		4) If there are any (whole) days between begin-day and end-day, compute total of how many billable hours in each of these days
'
'		5) Add results from (2) (3) and (4) to give final total of how many billable quarter hours
'
'The following line of code is a flag which controls whether debugging trace statements are displayed when the function is invoked

debug_mode = FALSE

' if the incoming parameters are not presented in standard date/time format, insert code here to re-shape them accordingly
BeginningValue = StartDateTime
EndingValue = EndDateTime
if debug_mode then response.write "<p><b>Starting Value</b> is " & BeginningValue & "<br>"
if debug_mode then response.write "<b>Ending Value</b> is " & EndingValue & "<P>"
%>
<!-- #INCLUDE FILE="../Language/ArrayHoursDuration.inc" -->
<%

' next we compute total number of billable quarters for each day of the week and stash in a counter for later use
dim daytotal(8)

daytotal(1) = 0
daytotal(2) = 0
daytotal(3) = 0
daytotal(4) = 0
daytotal(5) = 0
daytotal(6) = 0
daytotal(7) = 0

For i = 1 to 7
    workingDayVector = dayvector(i)
		For j = 1 to 96
			 if mid(workingDayVector,j,1) = "1" then
				daytotal(i) = daytotal(i) + 1
			 end if
		Next
       if debug_mode then  response.write "Total quarters of billable time on a " & dayofweek(i) & " is " & daytotal(i) & "<br>"

Next



TotQuarterCounter = 0

If not IsDate(StartDateTime) Then
    response.write "Starting Data Value Invalid<p>"
    response.flush
    response.end
end if

If not IsDate(EndDateTime) Then
    response.write "Ending Data Value Invalid<p>"
    response.flush
    response.end
end if

if  BeginningValue > EndingValue then
ComputeWeekdayHoursDuration = 0  ' Beginning is after start then just return 0
     Exit Function
'    response.write "Ending Date is before Starting Date - not valid<p>"
'    response.flush
'    response.end
end if



For i = 1 to 7 ' make sure the length of each vector is exactly 96 characters long
  length = len(dayvector(i))
  if length <> 96 then
     response.write "DayVector " & i  & " is not 96 characters long<p>"
     response.flush
     response.end
  end if
next


' we need to know how many days there are between beginning and end - especialy if it is zero so we don't count the hours on that day twice
ElapsedDays = DateDiff("d", BeginningValue, EndingValue)
if debug_mode then response.write "Elapsed Days between beginning and end is " & ElapsedDays & "<P>"



' -------------------------------- COMPUTE THE QUARTER HOURS OF THE BEGINNING DAY --------------------------------

' First we need to compute the ordinal (1-96) of Hrs/Mins part of the Beginning Value

StartingHour = DatePart("h",BeginningValue)

StartingMins = DatePart("n",BeginningValue)

if debug_mode then Response.write "The Starting hour is " & StartingHour & "<br>"
if debug_mode then Response.write "The Starting mins is " & StartingMins & "<br>"


StartingQuarter = ConvertTimeToQuarters(StartingHour,StartingMins)


if debug_mode then Response.write "<b>Therefore</b> the Starting quarter is " & StartingQuarter & "<br>"
EndingHour = DatePart("h",EndingValue)
EndingMins = DatePart("n",EndingValue)

If ElapsedDays = 0 Then
    limit = ConvertTimeToQuarters(EndingHour,EndingMins)
    if debug_mode then response.write "<b>And because</b> begin-day = end-day, the ending quarter is set to " & limit & "<p>"
else
    limit = 96
    if debug_mode then response.write "<b>And because</b> begin-day NOT =  end-day, the ending quarter is set to 96<P>"
end if

' now we go get the dayvector(n) for the starting day of week and put it in workingDayVector so we can loop thru it counting how many billable qurters on the beginning day


Beginningweekday = weekday(BeginningValue)
workingDayVector = dayvector(Beginningweekday)

if debug_mode then response.write "<pre>the vector for this day is    -" & workingDayVector & "- which is for " & dayofweek(Beginningweekday) & "<br>"
if debug_mode then response.write "the alignment for this day is  0        1         2         3         4         5         6         7         8         9       <br>"
if debug_mode then response.write "                               123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456 </pre><P>"
StartDayQuarterCounter = 0

' if the starting day is the same as the ending day, then we only scan the vector for ones from start-time to end-time, otherwise we scan from start-time to end-of-day = 96



'-------------------CAUTION - FOLLOWING LINE NEEDS CHECKING!!!!!!!!!!!!! midnight is an end-case !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!-------------

For i = startingquarter  to limit -1 ' note that on the beginning day we count from beginninning quarter upwards until the end of day (or end quarter if start-day = end-day)
     if mid(workingDayVector,i,1) = "1" then
        if debug_mode then response.write "start = " & startingquarter & " limit = " & limit & " i=" & i & " adding 1<br>"
        StartDayQuarterCounter = StartDayQuarterCounter + 1
     end if
Next
 if debug_mode then Response.write "<font color=red>the number of quarters to be charged on the beginning day is " & StartDayQuarterCounter & "</font><P>"

' -------------------------------- COMPUTE THE QUARTER HOURS OF THE ENDING DAY --------------------------------

If ElapsedDays = 0 Then
    EndingDayQuarterCounter = 0
    if debug_mode then Response.write "Because start-day = end-day, the number of quarters to be charged on the ending day is zero <P>"
else
		' First we need to compute the ordinal (1-96) of Hrs/Mins part of the Ending Value

		EndingHour = DatePart("h",EndingValue)

		EndingMins = DatePart("n",EndingValue)

		if debug_mode then Response.write "The Ending hour is " & EndingHour & "<br>"
		if debug_mode then Response.write "The Ending mins is " & EndingMins & "<br>"


		EndingQuarter = ConvertTimeToQuarters(EndingHour,EndingMins)


		if debug_mode then Response.write "<b>Therefore</b> the Ending quarter is " & EndingQuarter & " minus 1 because we don't count the last quarter at the end of the time-span, we count 'up to' and not 'thru'<P>"

		' now we go get the dayvector(n) for the ending day of week and put it in workingDayVector so we can loop thru it counting how many billable qurters on the beginning day


		EndingWeekDay = weekday(EndingValue)



		workingDayVector = dayvector(endingweekday)

		if debug_mode then response.write "<pre>the vector for this day is    -" & workingDayVector & "- which is for " & dayofweek(endingweekday) & "<br>"
		if debug_mode then response.write "the alignment for this day is  0        1         2         3         4         5         6         7         8         9       <br>"
		if debug_mode then response.write "                               123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456 </pre><P>"

		EndingDayQuarterCounter = 0

		For i = 1 to endingquarter - 1 ' note that on the ending day we count from beginninning of day until the ending quarter minus 1 (we don't count the final quarter as we are computing "up to" the end
			 if mid(workingDayVector,i,1) = "1" then
			    if debug_mode then response.write "start = 1 "  & " ending at  = " & endingquarter - 1 & " i=" & i & " adding 1<br>"
				EndingDayQuarterCounter = EndingDayQuarterCounter + 1
			 end if
		Next
		 if debug_mode then Response.write "<font color=red>the number of quarters to be charged on the ending day is " & EndingDayQuarterCounter & "</font><P>"
end if

' -------------------------------- COMPUTE THE QUARTER HOURS FOR EACH OF THE INTERVENING WHOLE DAYS BETWEEN BEGINNING AND END DAYS --------------------------------

if elapsedDays < 2 then
    ElapsedDaysQuarterCounter = 0
    if debug_mode then Response.write "Because there are no intervening days between beginning day and end day, the number of quarters to be charged during 'intervening' days is zero <P>"
else
    ElapsedDaysQuarterCounter = 0
    thisday = DateAdd("d",1,BeginningValue) ' start by kicking up to the first day after the beginning day

    Do while DateDiff("d", thisday, EndingValue) <> 0  ' and now loop thru successive days counting billable hours in each until we hit ending day
        thisweekday = weekday(thisday)
        if debug_mode then response.write "Adding " & Daytotal(thisweekday) & " for " & dayofweek(thisweekday) & " on the date " & thisday & "<br>"
        ElapsedDaysQuarterCounter = ElapsedDaysQuarterCounter + Daytotal(thisweekday)
        thisday = DateAdd("d",1,thisday) ' bump the day by 1 and loop again
    Loop
    if debug_mode then response.write "<font color=red> the number of quarters to be charged for intermediate full days is " & ElapsedDaysQuarterCounter & "</font><P>"
end if

' ------------------------------------------------------------------WRAP UP ---------------------------------------------------------------

final_total = StartDayQuarterCounter  + EndingDayQuarterCounter + ElapsedDaysQuarterCounter
if debug_mode then response.write "<font size=+1 color=red><b>Total quarter hours is " & final_total & "</font></b><p>"
ComputeWeekdayHoursDuration = final_total

End Function

Function ConvertTimeToQuarters(HH,MM)
QuarterIncrement = 0
If MM > 15 then QuarterIncrement = 1
If MM > 30 then QuarterIncrement = 2
If MM > 45 then QuarterIncrement = 3

ConvertTimeToQuarters = (HH*4) + QuarterIncrement + 1 ' The final +1 is to make the output range go 1-96 instead of 0-95

End Function
%>

